"use strict";

let cv, gl;

let program;

let vertices;
let vertexBuffer;

let lastFrame = Date.now();

async function init() {
	cv = document.getElementById("cv");
	gl = cv.getContext("webgl");

	if (!gl) {
		window.alert("WebGL not supported");
	}

	console.log("compiling shaders...");

	console.log("compiling vertex shader...");
	let vertexShader = await fetchAndCompileShader(gl, gl.VERTEX_SHADER, "shaders/vertex.glsl");

	console.log("compiling fragment shader...");
	let fragmentShader = await fetchAndCompileShader(gl, gl.FRAGMENT_SHADER, "shaders/fragment.glsl");

	console.log("linking shader program...");
	program = linkShaderProgram(gl, vertexShader, fragmentShader);

	/**
	 * -0.35              0.35
	 * |     -0.2    0.2  |
	 * |    |        |    |
	 * 
	 * +----+        +----+    ---  0.6
	 * |    |        |    |    
	 * |    |        |    |
	 * |    +--------+    |    ---  0.1
	 * |                  |
	 * |    +--------+    |    --- -0.1
	 * |    |        |    |
	 * |    |        |    |
	 * +----+        +----+    --- -0.6
	 *
	 * +------------------+    --- -0.7
	 * |                  |
	 * +------------------+    --- -0.9
	 *   
	 */    

	vertices = [
		// X  // Y

		// left vertical bar
		-0.35, -0.6,
		-0.35, 0.6,
		-0.2, 0.6,

		-0.35, -0.6,
		-0.2, 0.6,
		-0.2, -0.6,

		// right vertical bar
		0.35, -0.6,
		0.35, 0.6,
		0.2, 0.6,

		0.35, -0.6,
		0.2, 0.6,
		0.2, -0.6,

		// middle bar
		-0.2, -0.1,
		-0.2, 0.1,
		0.2, 0.1,

		-0.2, -0.1,
		0.2, 0.1,
		0.2, -0.1,

		// bottom bar
		-0.35, -0.7,
		-0.35, -0.9,
		0.35, -0.9,

		-0.35, -0.7,
		0.35, -0.9,
		0.35, -0.7
	];

	console.log("creating vertex buffer");

	vertexBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices), gl.STATIC_DRAW);

	// get location of vertPosition attribute
	let positionAttribLocation = gl.getAttribLocation(program, "vertPosition");

	gl.vertexAttribPointer(
		positionAttribLocation, // which attribute is read
		2, // number of values to read
		gl.FLOAT, // type of values
		gl.FALSE, // whether to normalize
		2 * Float32Array.BYTES_PER_ELEMENT, // size of individual vertex / distance between values to read
		0 // offset where to start reading
	);

	// enable attribute
	gl.enableVertexAttribArray(positionAttribLocation);

	// unbind buffer
	gl.bindBuffer(gl.ARRAY_BUFFER, null);

	// set clear colour
	gl.clearColor(0.1, 0.1, 0.1, 1.0);

	// start drawing
	requestAnimationFrame(draw);
}

let frameCount = 0;
let lastStatUpdate = Date.now();

function updateStats(deltaTime) {
	frameCount++;

	let now = Date.now();
	let timeSinceLastUpdate = (now - lastStatUpdate) / 1000;

	if (timeSinceLastUpdate < 0.5) return;

	lastStatUpdate = now;

	let fpsDisplay = document.getElementById("fps");
	fpsDisplay.innerText = Math.round(frameCount / timeSinceLastUpdate)

	let msDisplay = document.getElementById("ms");
	msDisplay.innerText = deltaTime * 1000 + "ms";

	frameCount = 0;
}

let hue = 0;

async function draw() {
	let now = Date.now();
	let deltaTime = (now - lastFrame) / 1000;

	hue += deltaTime / 5;
	if (hue > 1) hue -= 1;

	updateStats(deltaTime);

	gl.clear(gl.COLOR_BUFFER_BIT);

	gl.useProgram(program);

	let colorLocation = gl.getUniformLocation(program, "color");
	gl.uniform3f(colorLocation, ...hslToRgb(hue, 1, 0.5));

	gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);

	gl.drawArrays(gl.TRIANGLES, 0, vertices.length / 2);

	lastFrame = now;
	requestAnimationFrame(draw);
}

function hexColorToFloatArray(hex) {
	let r = parseInt(hex.substring(0, 2), 16) / 255;
	let g = parseInt(hex.substring(2, 4), 16) / 255;
	let b = parseInt(hex.substring(4, 6), 16) / 255;

	return [r, g, b];
}

function hslToRgb(h, s, l) {
	let r, g, b;

	if (s == 0) {
		r = g = b = l;
	} else {
		let hue2rgb = function hue2rgb(p, q, t) {
			if (t < 0) t += 1;
			if (t > 1) t -= 1;
			if (t < 1 / 6) return p + (q - p) * 6 * t;
			if (t < 1 / 2) return q;
			if (t < 2 / 3) return p + (q - p) * (2 / 3 - t) * 6;
			return p;
		}

		let q = l < 0.5 ? l * (1 + s) : l + s - l * s;
		let p = 2 * l - q;

		r = hue2rgb(p, q, h + 1 / 3);
		g = hue2rgb(p, q, h);
		b = hue2rgb(p, q, h - 1 / 3);
	}

	return [r, g, b];
}

window.addEventListener("DOMContentLoaded", init);